require 'spec_helper'

Test 'Subledger' do

  let(:factory) { Subledger::Factory.create }

  Instance do
    subject { factory.client }

    Test 'Book' do
      # POST /books
      RespondsTo 'book.create' do
        ByReturning 'an instance of Subledger::Domain::ActiveBook' do
          subject.book.create( factory.book_args ).
                         must_be_instance_of Subledger::Domain::ActiveBook
        end
      end

      # GET /books/:id
      RespondsTo 'books.read' do
        let(:book) { factory.create_book :active }

        ByReturning 'a kind of Subledger::Domain::Book' do
          subject.books.read( :id => book.id ).
                          must_be_instance_of Subledger::Domain::ActiveBook
        end
      end

      # PATCH /books/:id
      RespondsTo 'books.update' do
        let(:book       ) { factory.create_book :active }
        let(:description) { 'updated description' }
        let(:args       ) { book.attributes }
        let(:update_args) { args.merge :description => description,
                                       :version     => 2 }

        ByReturning 'an updated kind of Subledger::Domain::Book' do
          patched = subject.books.update update_args

          patched.must_be_instance_of Subledger::Domain::ActiveBook
          patched.description.must_equal description
        end
      end

      # GET /books ? before   = :description & limit = :limit & state = :state
      #              ending
      #              starting
      #              after
      #              preceding = :book_id & limit = :limit & state = :state
      #              following
      [ :archived, :active ].each do |state|
        Subledger.scenarios.each do |scenario|
          action       = scenario.keys.first
          expectations = scenario[action]

          value_key = if action == :preceding or action == :following
                        'id'
                      else
                        'description'
                      end

          RespondsTo "books.collect :state => #{state}, :action => #{action.inspect}" do
            let(:books) { factory.create_book_collection state }
            let(:book ) { books[2] }
            let(:limit) { 2 }

            if action == :preceding or action == :following
              let(:value) { book.id }
            else
              let(:value) { URI.escape book.description }
            end

            let(:args) { { :action          => action,
                           value_key.to_sym => value,
                           :state           => state,
                           :limit           => limit } }

            When 'not given a block' do
              let(:collection) { subject.books.collect args }

              ByReturning "a list of #{state} books" do
                collection.length.must_equal limit
              end
            end

            When 'given a block' do
              ByYielding "a list of #{state} books" do
                count = 0

                subject.books.collect( args ).each do |item|
                  expectation = expectations[count]

                  item.description.must_equal expectation

                  item.must_be_instance_of book.class

                  count += 1
                end

                count.must_equal limit
              end
            end
          end
        end
      end

      # POST /books/:id/archive
      [ :archived, :active ].each do |state|
        RespondsTo "#{state}_book_instance.archive" do
          let(:book) { factory.create_book state }

          ByReturning 'an instance of Subledger::Domain::ArchivedBook' do
            book.archive.must_be_instance_of Subledger::Domain::ArchivedBook
          end
        end
      end

      # POST /books/:id/activate
      [ :archived, :active ].each do |state|
        RespondsTo "#{state}_book_instance.activate" do
          let(:book) { factory.create_book state }

          ByReturning 'an instance of Subledger::Domain::ActiveBook' do
            book.activate.must_be_instance_of Subledger::Domain::ActiveBook
          end
        end
      end
    end
  end
end
