require 'spec_helper'

Test 'Subledger' do

  let(:factory) { Subledger::Factory.create }
  let(:amount ) { '1' }

  Instance do
    subject { factory.client }

    Test 'Report' do

      # POST /v2/orgs/:org_id/books/:book_id/reports
      RespondsTo 'report.create' do
        ByReturning 'an instance of Subledger::Domain::ActiveReport' do
          subject.report.create( factory.report_args ).
                     must_be_instance_of Subledger::Domain::ActiveReport
        end
      end

      # GET /v2/orgs/:org_id/books/:book_id/reports/:id
      [ :archived, :active ].each do |state|
        RespondsTo 'report.read' do
          let(:report) { factory.create_report state }

          ByReturning "an instance of #{state} report" do
            subject.report.read( :id => report.id ).
                       must_be_instance_of report.class
          end
        end
      end

      # PATCH /v2/orgs/:org_id/books/:book_id/reports/:id
      [ :archived, :active ].each do |state|
        RespondsTo 'reports.update' do
          let(:report   ) { factory.create_report state }
          let(:args       ) { report.attributes }
          let(:description) { 'updated description' }
          let(:update_args) { args.merge :description => description,
                                         :version     => 2 }

          ByReturning "an instance of #{state} report" do
            patched = subject.reports.update update_args

            patched.must_be_instance_of report.class
            patched.description.must_equal description
          end
        end
      end

      # GET /v2/orgs/:org_id/books/:book_id/reports ? before   = :description & limit = :limit & state = :state
      #                               ending
      #                               starting
      #                               after
      [ :active, :archived ].each do |state|
        Subledger.scenarios_order.each do |scenario|
          action       = scenario.keys.first
          expectations = scenario[action]

          RespondsTo "reports.collect :action => #{action}, :state => #{state}" do
            let(:reports) { factory.create_report_collection state }
            let(:item      ) { reports[2] }
            let(:limit     ) { 2 }
            let(:collection) { subject.reports.collect args }
            let(:klass     ) { item.class }

            let(:args) { { :action      => action,
                           :description => item.description,
                           :state       => state,
                           :limit       => limit } }

            When 'not given a block' do
              ByReturning "a list of #{state} reports" do
                collection.length.must_equal limit
              end
            end

            When 'given a block' do
              ByYielding "a list of #{state} reports" do
                count = 0

                collection.each do |report|
                  expectation = expectations[count]

                  report.description.must_equal expectation

                  report.must_be_instance_of klass

                  count += 1
                end

                count.must_equal limit
              end
            end
          end
        end
      end

      # GET /reports ? preceding = :book_id & limit = :limit & state = :state
      #                following = :book_id & limit = :limit & state = :state
      [ :active, :archived ].each do |state|
        Subledger.scenarios_id.each do |scenario|
          action       = scenario.keys.first
          expectations = scenario[action]

          RespondsTo "reports.collect :action => #{action}" do
            let(:reports) { factory.create_report_collection state }
            let(:item      ) { reports[2] }
            let(:limit     ) { 2 }
            let(:collection) { subject.reports.collect args }
            let(:klass     ) { item.class }

            let(:args) { { :action => action,
                           :id     => item.id,
                           :state  => state,
                           :limit  => limit } }

            When 'not given a block' do
              ByReturning "a list of #{state} reports" do
                collection.length.must_equal limit
              end
            end

            When 'given a block' do
              ByYielding "a list of #{state} reports" do
                count = 0

                collection.each do |report|
                  expectation = expectations[count]

                  report.description.must_equal expectation

                  report.must_be_instance_of klass

                  count += 1
                end

                count.must_equal limit
              end
            end
          end
        end
      end

      # GET /v1/orgs/:org_id/books/:book_id/reports ? before   = :description & limit = :limit & state = :state
      #                                               ending
      #                                               starting
      #                                               after
      # Test collects with large number of items
      [ :active ].each do |state|
        Subledger.scenarios_order.each do |scenario|
          action       = scenario.keys.first
          expectations = scenario[action]

          RespondsTo "reports.collect :action => #{action}, :state => #{state}" do
            let(:reports   ) { factory.create_large_report_collection state }

            let(:item      ) { reports[100] }
            let(:limit     ) { 100 }
            let(:collection) { subject.reports.collect args }
            let(:klass     ) { item.class }

            let(:args) { { :action      => action,
                           :description => item.description,
                           :state       => state,
                           :limit       => limit } }

            When 'not given a block' do
              ByReturning "a list of #{state} reports" do
                collection.length.must_equal limit
              end
            end

            When 'given a block' do
              ByYielding "a list of #{state} reports" do
                count = 0

                collection.each do |report|
                  report.must_be_instance_of klass

                  count += 1
                end

                count.must_equal limit
              end
            end
          end
        end
      end

      # GET /v1/orgs/:org_id/books/:book_id/reports ? preceding = :id & limit = :limit & state = :state
      #                                               following = :id
      # Test collects with large number of items
      [ :active ].each do |state|
        Subledger.scenarios_id.each do |scenario|
          action       = scenario.keys.first
          expectations = scenario[action]

          RespondsTo "reports.collect :action => #{action}, :state => #{state}" do
            let(:reports   ) { factory.create_large_report_collection state }

            let(:item      ) { reports[100] }
            let(:limit     ) { 100 }
            let(:collection) { subject.reports.collect args }
            let(:klass     ) { item.class }

            let(:args) { { :action => action,
                           :id     => item.id,
                           :state  => state,
                           :limit  => limit } }

            When 'not given a block' do
              ByReturning "a list of #{state} reports" do
                collection.length.must_equal limit
              end
            end

            When 'given a block' do
              ByYielding "a list of #{state} reports" do
                count = 0

                collection.each do |report|
                  report.must_be_instance_of klass

                  count += 1
                end

                count.must_equal limit
              end
            end
          end
        end
      end

      # POST /v1/orgs/:org_id/books/:book_id/reports/:id/archive
      RespondsTo 'report_instance.archive' do
        ByReturning 'an instance of Subledger::Domain::ArchivedReport' do
          factory.create_report(:active).archive.
                     must_be_instance_of Subledger::Domain::ArchivedReport
        end
      end

      # POST /v2/orgs/:org_id/books/:book_id/archived_reports/:id/activate
      RespondsTo 'archived_report_instance.activate' do
        ByReturning 'an instance of Subledger::Domain::ActiveReport' do
          factory.create_report(:archived).activate.
                     must_be_instance_of Subledger::Domain::ActiveReport
        end
      end

      # GET /v2/orgs/:org_id/books/:book_id/reports/:id/categories
      if Subledger::STORE == :api
        RespondsTo 'report_instance.categories' do
          ByRaising 'a Store::ReportError' do
            lambda do
              factory.create_report(:active).categories
            end.must_raise Subledger::Store::ReportError
          end
        end
      else
        RespondsTo 'report_instance.categories' do
          ByReturning 'a collection of Subledger::Domain::ActiveCategories' do
            factory.create_report(:active).categories.
                       must_be_instance_of Array
          end
        end
      end

      # POST /v2/orgs/:org_id/books/:book_id/reports/:id/render
      RespondsTo 'report_instance.render :at => Time' do
        let(:at) { Time.now.utc }

        ByReturning 'an instance of Subledger::Domain::BuildingReportRendering' do
          factory.create_report(:active).render(:at => at).
                     must_be_instance_of Subledger::Domain::BuildingReportRendering
        end
      end

      # GET /v2/orgs/:org_id/books/:book_id/report_renderings/:id
      RespondsTo 'to_json' do
        let(:report          ) { factory.loaded_report }
        let(:at              ) { Time.now }
        let(:report_rendering) { report.render :at => at }
        let(:rr_json         ) { report_rendering.to_json }
        let(:rr_hash         ) { MultiJson.load rr_json }

        before { report.render :at => at }

        ByReturning 'a REST report' do
          rr_hash.must_be_instance_of Hash
        end
      end
    end
  end
end
